@@ -4,12 +4,13 @@
 require 'date_core'
 
 class Date
-  VERSION = '2.0.2' # :nodoc:
+
+  def infinite?
+    false
+  end
 
   class Infinity < Numeric # :nodoc:
 
-    include Comparable
-
     def initialize(d=1) @d = d <=> 0 end
 
     def d() @d end
