@@ -2,13 +2,16 @@
 #
 #   irb/workspace-binding.rb -
 #   	$Release Version: 0.9.6$
-#   	$Revision: 60900 $
+#   	$Revision$
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
 #
 #
+
+require "delegate"
+
 module IRB # :nodoc:
   class WorkSpace
     # Creates a new workspace.
@@ -48,18 +51,24 @@ EOF
           end
           @binding = BINDING_QUEUE.pop
 
-        when 3	# binding in function on TOPLEVEL_BINDING(default)
-          @binding = eval("def irb_binding; private; binding; end; irb_binding",
+        when 3	# binding in function on TOPLEVEL_BINDING
+          @binding = eval("self.class.remove_method(:irb_binding) if defined?(irb_binding); private; def irb_binding; binding; end; irb_binding",
                           TOPLEVEL_BINDING,
                           __FILE__,
                           __LINE__ - 3)
+        when 4  # binding is a copy of TOPLEVEL_BINDING (default)
+          @binding = TOPLEVEL_BINDING.dup
         end
       end
+
       if main.empty?
         @main = eval("self", @binding)
       else
         @main = main[0]
-        IRB.conf[:__MAIN__] = @main
+      end
+      IRB.conf[:__MAIN__] = @main
+
+      unless main.empty?
         case @main
         when Module
           @binding = eval("IRB.conf[:__MAIN__].module_eval('binding', __FILE__, __LINE__)", @binding, __FILE__, __LINE__)
@@ -67,11 +76,33 @@ EOF
           begin
             @binding = eval("IRB.conf[:__MAIN__].instance_eval('binding', __FILE__, __LINE__)", @binding, __FILE__, __LINE__)
           rescue TypeError
-            IRB.fail CantChangeBinding, @main.inspect
+            fail CantChangeBinding, @main.inspect
           end
         end
       end
-      eval("_=nil", @binding)
+
+      case @main
+      when Object
+        use_delegator = @main.frozen?
+      else
+        use_delegator = true
+      end
+
+      if use_delegator
+        @main = SimpleDelegator.new(@main)
+        IRB.conf[:__MAIN__] = @main
+        @main.singleton_class.class_eval do
+          private
+          define_method(:exit) do |*a, &b|
+            # Do nothing, will be overridden
+          end
+          define_method(:binding, Kernel.instance_method(:binding))
+          define_method(:local_variables, Kernel.instance_method(:local_variables))
+        end
+        @binding = eval("IRB.conf[:__MAIN__].instance_eval('binding', __FILE__, __LINE__)", @binding, *@binding.source_location)
+      end
+
+      @binding.local_variable_set(:_, nil)
     end
 
     # The Binding of this workspace
@@ -85,48 +116,66 @@ EOF
       eval(statements, @binding, file, line)
     end
 
+    def local_variable_set(name, value)
+      @binding.local_variable_set(name, value)
+    end
+
+    def local_variable_get(name)
+      @binding.local_variable_get(name)
+    end
+
     # error message manipulator
     def filter_backtrace(bt)
+      return nil if bt =~ /\/irb\/.*\.rb/
+      return nil if bt =~ /\/irb\.rb/
+      return nil if bt =~ /tool\/lib\/.*\.rb|runner\.rb/ # for tests in Ruby repository
       case IRB.conf[:CONTEXT_MODE]
-      when 0
-        return nil if bt =~ /\(irb_local_binding\)/
       when 1
-        if(bt =~ %r!/tmp/irb-binding! or
-            bt =~ %r!irb/.*\.rb! or
-            bt =~ /irb\.rb/)
-          return nil
-        end
-      when 2
-        return nil if bt =~ /irb\/.*\.rb/
-        return nil if bt =~ /irb\.rb/
+        return nil if bt =~ %r!/tmp/irb-binding!
       when 3
-        return nil if bt =~ /irb\/.*\.rb/
-        return nil if bt =~ /irb\.rb/
         bt = bt.sub(/:\s*in `irb_binding'/, '')
       end
       bt
     end
 
     def code_around_binding
-      file, pos = @binding.eval('[__FILE__, __LINE__]')
+      if @binding.respond_to?(:source_location)
+        file, pos = @binding.source_location
+      else
+        file, pos = @binding.eval('[__FILE__, __LINE__]')
+      end
 
-      unless defined?(::SCRIPT_LINES__[file]) && lines = ::SCRIPT_LINES__[file]
+      if defined?(::SCRIPT_LINES__[file]) && lines = ::SCRIPT_LINES__[file]
+        code = ::SCRIPT_LINES__[file].join('')
+      else
         begin
-          lines = File.readlines(file)
+          code = File.read(file)
         rescue SystemCallError
           return
         end
       end
+
+      # NOT using #use_colorize? of IRB.conf[:MAIN_CONTEXT] because this method may be called before IRB::Irb#run
+      use_colorize = IRB.conf.fetch(:USE_COLORIZE, true)
+      if use_colorize
+        lines = Color.colorize_code(code).lines
+      else
+        lines = code.lines
+      end
       pos -= 1
 
       start_pos = [pos - 5, 0].max
       end_pos   = [pos + 5, lines.size - 1].min
 
-      fmt = " %2s %#{end_pos.to_s.length}d: %s"
+      if use_colorize
+        fmt = " %2s #{Color.colorize("%#{end_pos.to_s.length}d", [:BLUE, :BOLD])}: %s"
+      else
+        fmt = " %2s %#{end_pos.to_s.length}d: %s"
+      end
       body = (start_pos..end_pos).map do |current_pos|
         sprintf(fmt, pos == current_pos ? '=>' : '', current_pos + 1, lines[current_pos])
       end.join("")
-      "\nFrom: #{file} @ line #{pos + 1} :\n\n#{body}\n"
+      "\nFrom: #{file} @ line #{pos + 1} :\n\n#{body}#{Color.clear if use_colorize}\n"
     end
 
     def IRB.delete_caller
