@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 #
 # = net/ftp.rb - FTP Client Library
 #
@@ -17,6 +18,11 @@
 require "socket"
 require "monitor"
 require "net/protocol"
+require "time"
+begin
+  require "openssl"
+rescue LoadError
+end
 
 module Net
 
@@ -71,19 +77,25 @@ module Net
   # - #rename
   # - #delete
   #
-  class FTP
+  class FTP < Protocol
     include MonitorMixin
+    if defined?(OpenSSL::SSL)
+      include OpenSSL
+      include SSL
+    end
 
     # :stopdoc:
+    VERSION = "0.1.1"
     FTP_PORT = 21
     CRLF = "\r\n"
     DEFAULT_BLOCKSIZE = BufferedIO::BUFSIZE
+    @@default_passive = true
     # :startdoc:
 
     # When +true+, transfers are performed in binary mode.  Default: +true+.
     attr_reader :binary
 
-    # When +true+, the connection is in passive mode.  Default: +false+.
+    # When +true+, the connection is in passive mode.  Default: +true+.
     attr_accessor :passive
 
     # When +true+, all traffic to and from the server is written
@@ -100,6 +112,13 @@ module Net
     # Net::OpenTimeout exception. The default value is +nil+.
     attr_accessor :open_timeout
 
+    # Number of seconds to wait for the TLS handshake. Any number
+    # may be used, including Floats for fractional seconds. If the FTP
+    # object cannot complete the TLS handshake in this many seconds, it
+    # raises a Net::OpenTimeout exception. The default value is +nil+.
+    # If +ssl_handshake_timeout+ is +nil+, +open_timeout+ is used instead.
+    attr_accessor :ssl_handshake_timeout
+
     # Number of seconds to wait for one block to be read (via one read(2)
     # call). Any number may be used, including Floats for fractional
     # seconds. If the FTP object cannot read data in this many seconds,
@@ -122,44 +141,135 @@ module Net
     # The server's last response.
     attr_reader :last_response
 
+    # When +true+, connections are in passive mode per default.
+    # Default: +true+.
+    def self.default_passive=(value)
+      @@default_passive = value
+    end
+
+    # When +true+, connections are in passive mode per default.
+    # Default: +true+.
+    def self.default_passive
+      @@default_passive
+    end
+
     #
     # A synonym for <tt>FTP.new</tt>, but with a mandatory host parameter.
     #
     # If a block is given, it is passed the +FTP+ object, which will be closed
     # when the block finishes, or when an exception is raised.
     #
-    def FTP.open(host, user = nil, passwd = nil, acct = nil)
+    def FTP.open(host, *args)
       if block_given?
-        ftp = new(host, user, passwd, acct)
+        ftp = new(host, *args)
         begin
           yield ftp
         ensure
           ftp.close
         end
       else
-        new(host, user, passwd, acct)
+        new(host, *args)
       end
     end
 
+    # :call-seq:
+    #    Net::FTP.new(host = nil, options = {})
     #
     # Creates and returns a new +FTP+ object. If a +host+ is given, a connection
-    # is made. Additionally, if the +user+ is given, the given user name,
-    # password, and (optionally) account are used to log in.  See #login.
+    # is made.
     #
-    def initialize(host = nil, user = nil, passwd = nil, acct = nil)
+    # +options+ is an option hash, each key of which is a symbol.
+    #
+    # The available options are:
+    #
+    # port::      Port number (default value is 21)
+    # ssl::       If +options+[:ssl] is true, then an attempt will be made
+    #             to use SSL (now TLS) to connect to the server.  For this
+    #             to work OpenSSL [OSSL] and the Ruby OpenSSL [RSSL]
+    #             extensions need to be installed.  If +options+[:ssl] is a
+    #             hash, it's passed to OpenSSL::SSL::SSLContext#set_params
+    #             as parameters.
+    # private_data_connection::  If true, TLS is used for data connections.
+    #                            Default: +true+ when +options+[:ssl] is true.
+    # username::  Username for login.  If +options+[:username] is the string
+    #             "anonymous" and the +options+[:password] is +nil+,
+    #             "anonymous@" is used as a password.
+    # password::  Password for login.
+    # account::   Account information for ACCT.
+    # passive::   When +true+, the connection is in passive mode. Default:
+    #             +true+.
+    # open_timeout::  Number of seconds to wait for the connection to open.
+    #                 See Net::FTP#open_timeout for details.  Default: +nil+.
+    # read_timeout::  Number of seconds to wait for one block to be read.
+    #                 See Net::FTP#read_timeout for details.  Default: +60+.
+    # ssl_handshake_timeout::  Number of seconds to wait for the TLS
+    #                          handshake.
+    #                          See Net::FTP#ssl_handshake_timeout for
+    #                          details.  Default: +nil+.
+    # debug_mode::  When +true+, all traffic to and from the server is
+    #               written to +$stdout+.  Default: +false+.
+    #
+    def initialize(host = nil, user_or_options = {}, passwd = nil, acct = nil)
       super()
+      begin
+        options = user_or_options.to_hash
+      rescue NoMethodError
+        # for backward compatibility
+        options = {}
+        options[:username] = user_or_options
+        options[:password] = passwd
+        options[:account] = acct
+      end
+      @host = nil
+      if options[:ssl]
+        unless defined?(OpenSSL::SSL)
+          raise "SSL extension not installed"
+        end
+        ssl_params = options[:ssl] == true ? {} : options[:ssl]
+        @ssl_context = SSLContext.new
+        @ssl_context.set_params(ssl_params)
+        if defined?(VerifyCallbackProc)
+          @ssl_context.verify_callback = VerifyCallbackProc
+        end
+        @ssl_context.session_cache_mode =
+          OpenSSL::SSL::SSLContext::SESSION_CACHE_CLIENT |
+          OpenSSL::SSL::SSLContext::SESSION_CACHE_NO_INTERNAL_STORE
+        @ssl_context.session_new_cb = proc {|sock, sess| @ssl_session = sess }
+        @ssl_session = nil
+        if options[:private_data_connection].nil?
+          @private_data_connection = true
+        else
+          @private_data_connection = options[:private_data_connection]
+        end
+      else
+        @ssl_context = nil
+        if options[:private_data_connection]
+          raise ArgumentError,
+            "private_data_connection can be set to true only when ssl is enabled"
+        end
+        @private_data_connection = false
+      end
       @binary = true
-      @passive = false
-      @debug_mode = false
+      if options[:passive].nil?
+        @passive = @@default_passive
+      else
+        @passive = options[:passive]
+      end
+      if options[:debug_mode].nil?
+        @debug_mode = false
+      else
+        @debug_mode = options[:debug_mode]
+      end
       @resume = false
-      @sock = NullSocket.new
+      @bare_sock = @sock = NullSocket.new
       @logged_in = false
-      @open_timeout = nil
-      @read_timeout = 60
+      @open_timeout = options[:open_timeout]
+      @ssl_handshake_timeout = options[:ssl_handshake_timeout]
+      @read_timeout = options[:read_timeout] || 60
       if host
-        connect(host)
-        if user
-          login(user, passwd, acct)
+        connect(host, options[:port] || FTP_PORT)
+        if options[:username]
+          login(options[:username], options[:password], options[:account])
         end
       end
     end
@@ -205,35 +315,49 @@ module Net
 
     # Obsolete
     def return_code # :nodoc:
-      $stderr.puts("warning: Net::FTP#return_code is obsolete and do nothing")
+      warn("Net::FTP#return_code is obsolete and do nothing", uplevel: 1)
       return "\n"
     end
 
     # Obsolete
     def return_code=(s) # :nodoc:
-      $stderr.puts("warning: Net::FTP#return_code= is obsolete and do nothing")
+      warn("Net::FTP#return_code= is obsolete and do nothing", uplevel: 1)
     end
 
     # Constructs a socket with +host+ and +port+.
     #
     # If SOCKSSocket is defined and the environment (ENV) defines
-    # SOCKS_SERVER, then a SOCKSSocket is returned, else a TCPSocket is
+    # SOCKS_SERVER, then a SOCKSSocket is returned, else a Socket is
     # returned.
     def open_socket(host, port) # :nodoc:
-      return Timeout.timeout(@open_timeout, Net::OpenTimeout) {
+      return Timeout.timeout(@open_timeout, OpenTimeout) {
         if defined? SOCKSSocket and ENV["SOCKS_SERVER"]
           @passive = true
-          sock = SOCKSSocket.open(host, port)
+          SOCKSSocket.open(host, port)
         else
-          sock = TCPSocket.open(host, port)
+          Socket.tcp(host, port)
         end
-        io = BufferedSocket.new(sock)
-        io.read_timeout = @read_timeout
-        io
       }
     end
     private :open_socket
 
+    def start_tls_session(sock)
+      ssl_sock = SSLSocket.new(sock, @ssl_context)
+      ssl_sock.sync_close = true
+      ssl_sock.hostname = @host if ssl_sock.respond_to? :hostname=
+      if @ssl_session &&
+          Process.clock_gettime(Process::CLOCK_REALTIME) < @ssl_session.time.to_f + @ssl_session.timeout
+        # ProFTPD returns 425 for data connections if session is not reused.
+        ssl_sock.session = @ssl_session
+      end
+      ssl_socket_connect(ssl_sock, @ssl_handshake_timeout || @open_timeout)
+      if @ssl_context.verify_mode != VERIFY_NONE
+        ssl_sock.post_connection_check(@host)
+      end
+      return ssl_sock
+    end
+    private :start_tls_session
+
     #
     # Establishes an FTP connection to host, optionally overriding the default
     # port. If the environment variable +SOCKS_SERVER+ is set, sets up the
@@ -245,8 +369,24 @@ module Net
         print "connect: ", host, ", ", port, "\n"
       end
       synchronize do
-        @sock = open_socket(host, port)
+        @host = host
+        @bare_sock = open_socket(host, port)
+        @sock = BufferedSocket.new(@bare_sock, read_timeout: @read_timeout)
         voidresp
+        if @ssl_context
+          begin
+            voidcmd("AUTH TLS")
+            ssl_sock = start_tls_session(@bare_sock)
+            @sock = BufferedSSLSocket.new(ssl_sock, read_timeout: @read_timeout)
+            if @private_data_connection
+              voidcmd("PBSZ 0")
+              voidcmd("PROT P")
+            end
+          rescue OpenSSL::SSL::SSLError, OpenTimeout
+            @sock.close
+            raise
+          end
+        end
       end
     end
 
@@ -301,16 +441,16 @@ module Net
 
     # Receive a section of lines until the response code's match.
     def getmultiline # :nodoc:
-      line = getline
-      buff = line
-      if line[3] == ?-
-          code = line[0, 3]
+      lines = []
+      lines << getline
+      code = lines.last.slice(/\A([0-9a-zA-Z]{3})-/, 1)
+      if code
+        delimiter = code + " "
         begin
-          line = getline
-          buff << "\n" << line
-        end until line[0, 3] == code and line[3] != ?-
+          lines << getline
+        end until lines.last.start_with?(delimiter)
       end
-      return buff << "\n"
+      return lines.join("\n") + "\n"
     end
     private :getmultiline
 
@@ -340,7 +480,7 @@ module Net
     # equal 2.
     def voidresp # :nodoc:
       resp = getresp
-      if resp[0] != ?2
+      if !resp.start_with?("2")
         raise FTPReplyError, resp
       end
     end
@@ -368,10 +508,10 @@ module Net
 
     # Constructs and send the appropriate PORT (or EPRT) command
     def sendport(host, port) # :nodoc:
-      af = (@sock.peeraddr)[0]
-      if af == "AF_INET"
+      remote_address = @bare_sock.remote_address
+      if remote_address.ipv4?
         cmd = "PORT " + (host.split(".") + port.divmod(256)).join(",")
-      elsif af == "AF_INET6"
+      elsif remote_address.ipv6?
         cmd = sprintf("EPRT |2|%s|%d|", host, port)
       else
         raise FTPProtoError, host
@@ -382,13 +522,13 @@ module Net
 
     # Constructs a TCPServer socket
     def makeport # :nodoc:
-      TCPServer.open(@sock.addr[3], 0)
+      Addrinfo.tcp(@bare_sock.local_address.ip_address, 0).listen
     end
     private :makeport
 
     # sends the appropriate command to enable a passive connection
     def makepasv # :nodoc:
-      if @sock.peeraddr[0] == "AF_INET"
+      if @bare_sock.remote_address.ipv4?
         host, port = parse227(sendcmd("PASV"))
       else
         host, port = parse229(sendcmd("EPSV"))
@@ -405,41 +545,46 @@ module Net
         conn = open_socket(host, port)
         if @resume and rest_offset
           resp = sendcmd("REST " + rest_offset.to_s)
-          if resp[0] != ?3
+          if !resp.start_with?("3")
             raise FTPReplyError, resp
           end
         end
         resp = sendcmd(cmd)
         # skip 2XX for some ftp servers
-        resp = getresp if resp[0] == ?2
-        if resp[0] != ?1
+        resp = getresp if resp.start_with?("2")
+        if !resp.start_with?("1")
           raise FTPReplyError, resp
         end
       else
         sock = makeport
         begin
-          sendport(sock.addr[3], sock.addr[1])
+          addr = sock.local_address
+          sendport(addr.ip_address, addr.ip_port)
           if @resume and rest_offset
             resp = sendcmd("REST " + rest_offset.to_s)
-            if resp[0] != ?3
+            if !resp.start_with?("3")
               raise FTPReplyError, resp
             end
           end
           resp = sendcmd(cmd)
           # skip 2XX for some ftp servers
-          resp = getresp if resp[0] == ?2
-          if resp[0] != ?1
+          resp = getresp if resp.start_with?("2")
+          if !resp.start_with?("1")
             raise FTPReplyError, resp
           end
-          conn = BufferedSocket.new(sock.accept)
-          conn.read_timeout = @read_timeout
+          conn, = sock.accept
           sock.shutdown(Socket::SHUT_WR) rescue nil
           sock.read rescue nil
         ensure
           sock.close
         end
       end
-      return conn
+      if @private_data_connection
+        return BufferedSSLSocket.new(start_tls_session(conn),
+                                     read_timeout: @read_timeout)
+      else
+        return BufferedSocket.new(conn, read_timeout: @read_timeout)
+      end
     end
     private :transfercmd
 
@@ -459,16 +604,16 @@ module Net
       resp = ""
       synchronize do
         resp = sendcmd('USER ' + user)
-        if resp[0] == ?3
+        if resp.start_with?("3")
           raise FTPReplyError, resp if passwd.nil?
           resp = sendcmd('PASS ' + passwd)
         end
-        if resp[0] == ?3
+        if resp.start_with?("3")
           raise FTPReplyError, resp if acct.nil?
           resp = sendcmd('ACCT ' + acct)
         end
       end
-      if resp[0] != ?2
+      if !resp.start_with?("2")
         raise FTPReplyError, resp
       end
       @welcome = resp
@@ -487,14 +632,12 @@ module Net
         with_binary(true) do
           begin
             conn = transfercmd(cmd, rest_offset)
-            loop do
-              data = conn.read(blocksize)
-              break if data == nil
+            while data = conn.read(blocksize)
               yield(data)
             end
-            conn.shutdown(Socket::SHUT_WR)
+            conn.shutdown(Socket::SHUT_WR) rescue nil
             conn.read_timeout = 1
-            conn.read
+            conn.read rescue nil
           ensure
             conn.close if conn
           end
@@ -514,14 +657,12 @@ module Net
         with_binary(false) do
           begin
             conn = transfercmd(cmd)
-            loop do
-              line = conn.gets
-              break if line == nil
+            while line = conn.gets
               yield(line.sub(/\r?\n\z/, ""), !line.match(/\n\z/).nil?)
             end
-            conn.shutdown(Socket::SHUT_WR)
+            conn.shutdown(Socket::SHUT_WR) rescue nil
             conn.read_timeout = 1
-            conn.read
+            conn.read rescue nil
           ensure
             conn.close if conn
           end
@@ -542,14 +683,18 @@ module Net
       end
       synchronize do
         with_binary(true) do
-          conn = transfercmd(cmd)
-          loop do
-            buf = file.read(blocksize)
-            break if buf == nil
-            conn.write(buf)
-            yield(buf) if block_given?
+          begin
+            conn = transfercmd(cmd)
+            while buf = file.read(blocksize)
+              conn.write(buf)
+              yield(buf) if block_given?
+            end
+            conn.shutdown(Socket::SHUT_WR) rescue nil
+            conn.read_timeout = 1
+            conn.read rescue nil
+          ensure
+            conn.close if conn
           end
-          conn.close
           voidresp
         end
       end
@@ -571,17 +716,21 @@ module Net
     def storlines(cmd, file) # :yield: line
       synchronize do
         with_binary(false) do
-          conn = transfercmd(cmd)
-          loop do
-            buf = file.gets
-            break if buf == nil
-            if buf[-2, 2] != CRLF
-              buf = buf.chomp + CRLF
+          begin
+            conn = transfercmd(cmd)
+            while buf = file.gets
+              if buf[-2, 2] != CRLF
+                buf = buf.chomp + CRLF
+              end
+              conn.write(buf)
+              yield(buf) if block_given?
             end
-            conn.write(buf)
-            yield(buf) if block_given?
+            conn.shutdown(Socket::SHUT_WR) rescue nil
+            conn.read_timeout = 1
+            conn.read rescue nil
+          ensure
+            conn.close if conn
           end
-          conn.close
           voidresp
         end
       end
@@ -601,29 +750,30 @@ module Net
     # chunks.
     #
     def getbinaryfile(remotefile, localfile = File.basename(remotefile),
-                      blocksize = DEFAULT_BLOCKSIZE) # :yield: data
+                      blocksize = DEFAULT_BLOCKSIZE, &block) # :yield: data
+      f = nil
       result = nil
       if localfile
         if @resume
           rest_offset = File.size?(localfile)
-          f = open(localfile, "a")
+          f = File.open(localfile, "a")
         else
           rest_offset = nil
-          f = open(localfile, "w")
+          f = File.open(localfile, "w")
         end
       elsif !block_given?
-        result = ""
+        result = String.new
       end
       begin
-        f.binmode if localfile
-        retrbinary("RETR " + remotefile.to_s, blocksize, rest_offset) do |data|
-          f.write(data) if localfile
-          yield(data) if block_given?
-          result.concat(data) if result
+        f&.binmode
+        retrbinary("RETR #{remotefile}", blocksize, rest_offset) do |data|
+          f&.write(data)
+          block&.(data)
+          result&.concat(data)
         end
         return result
       ensure
-        f.close if localfile
+        f&.close
       end
     end
 
@@ -634,23 +784,25 @@ module Net
     # If a block is supplied, it is passed the retrieved data one
     # line at a time.
     #
-    def gettextfile(remotefile, localfile = File.basename(remotefile)) # :yield: line
+    def gettextfile(remotefile, localfile = File.basename(remotefile),
+                    &block) # :yield: line
+      f = nil
       result = nil
       if localfile
-        f = open(localfile, "w")
+        f = File.open(localfile, "w")
       elsif !block_given?
-        result = ""
+        result = String.new
       end
       begin
-        retrlines("RETR " + remotefile) do |line, newline|
+        retrlines("RETR #{remotefile}") do |line, newline|
           l = newline ? line + "\n" : line
-          f.print(l) if localfile
-          yield(line, newline) if block_given?
-          result.concat(l) if result
+          f&.print(l)
+          block&.(line, newline)
+          result&.concat(l)
         end
         return result
       ensure
-        f.close if localfile
+        f&.close
       end
     end
 
@@ -683,13 +835,13 @@ module Net
       else
         rest_offset = nil
       end
-      f = open(localfile)
+      f = File.open(localfile)
       begin
         f.binmode
         if rest_offset
-          storbinary("APPE " + remotefile, f, blocksize, rest_offset, &block)
+          storbinary("APPE #{remotefile}", f, blocksize, rest_offset, &block)
         else
-          storbinary("STOR " + remotefile, f, blocksize, rest_offset, &block)
+          storbinary("STOR #{remotefile}", f, blocksize, rest_offset, &block)
         end
       ensure
         f.close
@@ -702,9 +854,9 @@ module Net
     # passing in the transmitted data one line at a time.
     #
     def puttextfile(localfile, remotefile = File.basename(localfile), &block) # :yield: line
-      f = open(localfile)
+      f = File.open(localfile)
       begin
-        storlines("STOR " + remotefile, f, &block)
+        storlines("STOR #{remotefile}", f, &block)
       ensure
         f.close
       end
@@ -740,7 +892,7 @@ module Net
     def nlst(dir = nil)
       cmd = "NLST"
       if dir
-        cmd = cmd + " " + dir
+        cmd = "#{cmd} #{dir}"
       end
       files = []
       retrlines(cmd) do |line|
@@ -756,40 +908,238 @@ module Net
     def list(*args, &block) # :yield: line
       cmd = "LIST"
       args.each do |arg|
-        cmd = cmd + " " + arg.to_s
+        cmd = "#{cmd} #{arg}"
+      end
+      lines = []
+      retrlines(cmd) do |line|
+        lines << line
       end
       if block
-        retrlines(cmd, &block)
-      else
-        lines = []
-        retrlines(cmd) do |line|
-          lines << line
-        end
-        return lines
+        lines.each(&block)
       end
+      return lines
     end
     alias ls list
     alias dir list
 
+    #
+    # MLSxEntry represents an entry in responses of MLST/MLSD.
+    # Each entry has the facts (e.g., size, last modification time, etc.)
+    # and the pathname.
+    #
+    class MLSxEntry
+      attr_reader :facts, :pathname
+
+      def initialize(facts, pathname)
+        @facts = facts
+        @pathname = pathname
+      end
+
+      standard_facts = %w(size modify create type unique perm
+                          lang media-type charset)
+      standard_facts.each do |factname|
+        define_method factname.gsub(/-/, "_") do
+          facts[factname]
+        end
+      end
+
+      #
+      # Returns +true+ if the entry is a file (i.e., the value of the type
+      # fact is file).
+      #
+      def file?
+        return facts["type"] == "file"
+      end
+
+      #
+      # Returns +true+ if the entry is a directory (i.e., the value of the
+      # type fact is dir, cdir, or pdir).
+      #
+      def directory?
+        if /\A[cp]?dir\z/.match(facts["type"])
+          return true
+        else
+          return false
+        end
+      end
+
+      #
+      # Returns +true+ if the APPE command may be applied to the file.
+      #
+      def appendable?
+        return facts["perm"].include?(?a)
+      end
+
+      #
+      # Returns +true+ if files may be created in the directory by STOU,
+      # STOR, APPE, and RNTO.
+      #
+      def creatable?
+        return facts["perm"].include?(?c)
+      end
+
+      #
+      # Returns +true+ if the file or directory may be deleted by DELE/RMD.
+      #
+      def deletable?
+        return facts["perm"].include?(?d)
+      end
+
+      #
+      # Returns +true+ if the directory may be entered by CWD/CDUP.
+      #
+      def enterable?
+        return facts["perm"].include?(?e)
+      end
+
+      #
+      # Returns +true+ if the file or directory may be renamed by RNFR.
+      #
+      def renamable?
+        return facts["perm"].include?(?f)
+      end
+
+      #
+      # Returns +true+ if the listing commands, LIST, NLST, and MLSD are
+      # applied to the directory.
+      #
+      def listable?
+        return facts["perm"].include?(?l)
+      end
+
+      #
+      # Returns +true+ if the MKD command may be used to create a new
+      # directory within the directory.
+      #
+      def directory_makable?
+        return facts["perm"].include?(?m)
+      end
+
+      #
+      # Returns +true+ if the objects in the directory may be deleted, or
+      # the directory may be purged.
+      #
+      def purgeable?
+        return facts["perm"].include?(?p)
+      end
+
+      #
+      # Returns +true+ if the RETR command may be applied to the file.
+      #
+      def readable?
+        return facts["perm"].include?(?r)
+      end
+
+      #
+      # Returns +true+ if the STOR command may be applied to the file.
+      #
+      def writable?
+        return facts["perm"].include?(?w)
+      end
+    end
+
+    CASE_DEPENDENT_PARSER = ->(value) { value }
+    CASE_INDEPENDENT_PARSER = ->(value) { value.downcase }
+    DECIMAL_PARSER = ->(value) { value.to_i }
+    OCTAL_PARSER = ->(value) { value.to_i(8) }
+    TIME_PARSER = ->(value, local = false) {
+      unless /\A(?<year>\d{4})(?<month>\d{2})(?<day>\d{2})
+            (?<hour>\d{2})(?<min>\d{2})(?<sec>\d{2})
+            (?:\.(?<fractions>\d+))?/x =~ value
+        raise FTPProtoError, "invalid time-val: #{value}"
+      end
+      usec = fractions.to_i * 10 ** (6 - fractions.to_s.size)
+      Time.public_send(local ? :local : :utc, year, month, day, hour, min, sec, usec)
+    }
+    FACT_PARSERS = Hash.new(CASE_DEPENDENT_PARSER)
+    FACT_PARSERS["size"] = DECIMAL_PARSER
+    FACT_PARSERS["modify"] = TIME_PARSER
+    FACT_PARSERS["create"] = TIME_PARSER
+    FACT_PARSERS["type"] = CASE_INDEPENDENT_PARSER
+    FACT_PARSERS["unique"] = CASE_DEPENDENT_PARSER
+    FACT_PARSERS["perm"] = CASE_INDEPENDENT_PARSER
+    FACT_PARSERS["lang"] = CASE_INDEPENDENT_PARSER
+    FACT_PARSERS["media-type"] = CASE_INDEPENDENT_PARSER
+    FACT_PARSERS["charset"] = CASE_INDEPENDENT_PARSER
+    FACT_PARSERS["unix.mode"] = OCTAL_PARSER
+    FACT_PARSERS["unix.owner"] = DECIMAL_PARSER
+    FACT_PARSERS["unix.group"] = DECIMAL_PARSER
+    FACT_PARSERS["unix.ctime"] = TIME_PARSER
+    FACT_PARSERS["unix.atime"] = TIME_PARSER
+
+    def parse_mlsx_entry(entry)
+      facts, pathname = entry.chomp.split(/ /, 2)
+      unless pathname
+        raise FTPProtoError, entry
+      end
+      return MLSxEntry.new(
+        facts.scan(/(.*?)=(.*?);/).each_with_object({}) {
+          |(factname, value), h|
+          name = factname.downcase
+          h[name] = FACT_PARSERS[name].(value)
+        },
+        pathname)
+    end
+    private :parse_mlsx_entry
+
+    #
+    # Returns data (e.g., size, last modification time, entry type, etc.)
+    # about the file or directory specified by +pathname+.
+    # If +pathname+ is omitted, the current directory is assumed.
+    #
+    def mlst(pathname = nil)
+      cmd = pathname ? "MLST #{pathname}" : "MLST"
+      resp = sendcmd(cmd)
+      if !resp.start_with?("250")
+        raise FTPReplyError, resp
+      end
+      line = resp.lines[1]
+      unless line
+        raise FTPProtoError, resp
+      end
+      entry = line.sub(/\A(250-| *)/, "")
+      return parse_mlsx_entry(entry)
+    end
+
+    #
+    # Returns an array of the entries of the directory specified by
+    # +pathname+.
+    # Each entry has the facts (e.g., size, last modification time, etc.)
+    # and the pathname.
+    # If a block is given, it iterates through the listing.
+    # If +pathname+ is omitted, the current directory is assumed.
+    #
+    def mlsd(pathname = nil, &block) # :yield: entry
+      cmd = pathname ? "MLSD #{pathname}" : "MLSD"
+      entries = []
+      retrlines(cmd) do |line|
+        entries << parse_mlsx_entry(line)
+      end
+      if block
+        entries.each(&block)
+      end
+      return entries
+    end
+
     #
     # Renames a file on the server.
     #
     def rename(fromname, toname)
-      resp = sendcmd("RNFR " + fromname)
-      if resp[0] != ?3
+      resp = sendcmd("RNFR #{fromname}")
+      if !resp.start_with?("3")
         raise FTPReplyError, resp
       end
-      voidcmd("RNTO " + toname)
+      voidcmd("RNTO #{toname}")
     end
 
     #
     # Deletes a file on the server.
     #
     def delete(filename)
-      resp = sendcmd("DELE " + filename)
-      if resp[0, 3] == "250"
+      resp = sendcmd("DELE #{filename}")
+      if resp.start_with?("250")
         return
-      elsif resp[0] == ?5
+      elsif resp.start_with?("5")
         raise FTPPermError, resp
       else
         raise FTPReplyError, resp
@@ -810,40 +1160,41 @@ module Net
           end
         end
       end
-      cmd = "CWD " + dirname
+      cmd = "CWD #{dirname}"
       voidcmd(cmd)
     end
 
+    def get_body(resp) # :nodoc:
+      resp.slice(/\A[0-9a-zA-Z]{3} (.*)$/, 1)
+    end
+    private :get_body
+
     #
     # Returns the size of the given (remote) filename.
     #
     def size(filename)
       with_binary(true) do
-        resp = sendcmd("SIZE " + filename)
-        if resp[0, 3] != "213"
+        resp = sendcmd("SIZE #{filename}")
+        if !resp.start_with?("213")
           raise FTPReplyError, resp
         end
-        return resp[3..-1].strip.to_i
+        return get_body(resp).to_i
       end
     end
 
-    MDTM_REGEXP = /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/  # :nodoc:
-
     #
     # Returns the last modification time of the (remote) file.  If +local+ is
     # +true+, it is returned as a local time, otherwise it's a UTC time.
     #
     def mtime(filename, local = false)
-      str = mdtm(filename)
-      ary = str.scan(MDTM_REGEXP)[0].collect {|i| i.to_i}
-      return local ? Time.local(*ary) : Time.gm(*ary)
+      return TIME_PARSER.(mdtm(filename), local)
     end
 
     #
     # Creates a remote directory.
     #
     def mkdir(dirname)
-      resp = sendcmd("MKD " + dirname)
+      resp = sendcmd("MKD #{dirname}")
       return parse257(resp)
     end
 
@@ -851,7 +1202,7 @@ module Net
     # Removes a remote directory.
     #
     def rmdir(dirname)
-      voidcmd("RMD " + dirname)
+      voidcmd("RMD #{dirname}")
     end
 
     #
@@ -868,10 +1219,10 @@ module Net
     #
     def system
       resp = sendcmd("SYST")
-      if resp[0, 3] != "215"
+      if !resp.start_with?("215")
         raise FTPReplyError, resp
       end
-      return resp[4 .. -1]
+      return get_body(resp)
     end
 
     #
@@ -891,10 +1242,16 @@ module Net
     #
     # Returns the status (STAT command).
     #
-    def status
-      line = "STAT" + CRLF
-      print "put: STAT\n" if @debug_mode
-      @sock.send(line, Socket::MSG_OOB)
+    # pathname:: when stat is invoked with pathname as a parameter it acts like
+    #            list but a lot faster and over the same tcp session.
+    #
+    def status(pathname = nil)
+      line = pathname ? "STAT #{pathname}" : "STAT"
+      if /[\r\n]/ =~ line
+        raise ArgumentError, "A line must not contain CR or LF"
+      end
+      print "put: #{line}\n" if @debug_mode
+      @sock.send(line + CRLF, Socket::MSG_OOB)
       return getresp
     end
 
@@ -905,9 +1262,9 @@ module Net
     # Use +mtime+ if you want a parsed Time instance.
     #
     def mdtm(filename)
-      resp = sendcmd("MDTM " + filename)
-      if resp[0, 3] == "213"
-        return resp[3 .. -1].strip
+      resp = sendcmd("MDTM #{filename}")
+      if resp.start_with?("213")
+        return get_body(resp)
       end
     end
 
@@ -946,6 +1303,41 @@ module Net
       voidcmd(cmd)
     end
 
+    #
+    # Issues a FEAT command
+    #
+    # Returns an array of supported optional features
+    #
+    def features
+      resp = sendcmd("FEAT")
+      if !resp.start_with?("211")
+        raise FTPReplyError, resp
+      end
+
+      feats = []
+      resp.split("\n").each do |line|
+        next if !line.start_with?(' ') # skip status lines
+
+        feats << line.strip
+      end
+
+      return feats
+    end
+
+    #
+    # Issues an OPTS command
+    # - name Should be the name of the option to set
+    # - params is any optional parameters to supply with the option
+    #
+    # example: option('UTF8', 'ON') => 'OPTS UTF8 ON'
+    #
+    def option(name, params = nil)
+      cmd = "OPTS #{name}"
+      cmd += " #{params}" if params
+
+      voidcmd(cmd)
+    end
+
     #
     # Closes the connection.  Further operations are impossible until you open
     # a new connection with #connect.
@@ -975,10 +1367,10 @@ module Net
     #
     # Returns host and port.
     def parse227(resp) # :nodoc:
-      if resp[0, 3] != "227"
+      if !resp.start_with?("227")
         raise FTPReplyError, resp
       end
-      if m = /\((?<host>\d+(,\d+){3}),(?<port>\d+,\d+)\)/.match(resp)
+      if m = /\((?<host>\d+(?:,\d+){3}),(?<port>\d+,\d+)\)/.match(resp)
         return parse_pasv_ipv4_host(m["host"]), parse_pasv_port(m["port"])
       else
         raise FTPProtoError, resp
@@ -991,12 +1383,12 @@ module Net
     #
     # Returns host and port.
     def parse228(resp) # :nodoc:
-      if resp[0, 3] != "228"
+      if !resp.start_with?("228")
         raise FTPReplyError, resp
       end
-      if m = /\(4,4,(?<host>\d+(,\d+){3}),2,(?<port>\d+,\d+)\)/.match(resp)
+      if m = /\(4,4,(?<host>\d+(?:,\d+){3}),2,(?<port>\d+,\d+)\)/.match(resp)
         return parse_pasv_ipv4_host(m["host"]), parse_pasv_port(m["port"])
-      elsif m = /\(6,16,(?<host>\d+(,(\d+)){15}),2,(?<port>\d+,\d+)\)/.match(resp)
+      elsif m = /\(6,16,(?<host>\d+(?:,\d+){15}),2,(?<port>\d+,\d+)\)/.match(resp)
         return parse_pasv_ipv6_host(m["host"]), parse_pasv_port(m["port"])
       else
         raise FTPProtoError, resp
@@ -1028,11 +1420,11 @@ module Net
     #
     # Returns host and port.
     def parse229(resp) # :nodoc:
-      if resp[0, 3] != "229"
+      if !resp.start_with?("229")
         raise FTPReplyError, resp
       end
       if m = /\((?<d>[!-~])\k<d>\k<d>(?<port>\d+)\k<d>\)/.match(resp)
-        return @sock.peeraddr[3], m["port"].to_i
+        return @bare_sock.remote_address.ip_address, m["port"].to_i
       else
         raise FTPProtoError, resp
       end
@@ -1044,27 +1436,10 @@ module Net
     #
     # Returns host and port.
     def parse257(resp) # :nodoc:
-      if resp[0, 3] != "257"
+      if !resp.start_with?("257")
         raise FTPReplyError, resp
       end
-      if resp[3, 2] != ' "'
-        return ""
-      end
-      dirname = ""
-      i = 5
-      n = resp.length
-      while i < n
-        c = resp[i, 1]
-        i = i + 1
-        if c == '"'
-          if i > n or resp[i, 1] != '"'
-            break
-          end
-          i = i + 1
-        end
-        dirname = dirname + c
-      end
-      return dirname
+      return resp.slice(/"(([^"]|"")*)"/, 1).to_s.gsub(/""/, '"')
     end
     private :parse257
 
@@ -1086,7 +1461,7 @@ module Net
     end
 
     class BufferedSocket < BufferedIO
-      [:addr, :peeraddr, :send, :shutdown].each do |method|
+      [:local_address, :remote_address, :addr, :peeraddr, :send, :shutdown].each do |method|
         define_method(method) { |*args|
           @io.__send__(method, *args)
         }
@@ -1094,11 +1469,11 @@ module Net
 
       def read(len = nil)
         if len
-          s = super(len, "", true)
+          s = super(len, String.new, true)
           return s.empty? ? nil : s
         else
-          result = ""
-          while s = super(DEFAULT_BLOCKSIZE, "", true)
+          result = String.new
+          while s = super(DEFAULT_BLOCKSIZE, String.new, true)
             break if s.empty?
             result << s
           end
@@ -1119,6 +1494,37 @@ module Net
         return line
       end
     end
+
+    if defined?(OpenSSL::SSL::SSLSocket)
+      class BufferedSSLSocket <  BufferedSocket
+        def initialize(*args, **options)
+          super
+          @is_shutdown = false
+        end
+
+        def shutdown(*args)
+          # SSL_shutdown() will be called from SSLSocket#close, and
+          # SSL_shutdown() will send the "close notify" alert to the peer,
+          # so shutdown(2) should not be called.
+          @is_shutdown = true
+        end
+
+        def send(mesg, flags, dest = nil)
+          # Ignore flags and dest.
+          @io.write(mesg)
+        end
+
+        private
+
+        def rbuf_fill
+          if @is_shutdown
+            raise EOFError, "shutdown has been called"
+          else
+            super
+          end
+        end
+      end
+    end
     # :startdoc:
   end
 end
