@@ -12,7 +12,6 @@ require 'rubygems/version_option'
 # See `gem help install`
 
 class Gem::Commands::InstallCommand < Gem::Command
-
   attr_reader :installed_specs # :nodoc:
 
   include Gem::VersionOption
@@ -218,7 +217,7 @@ You can use `i` command instead of `install`.
       gem_version ||= options[:version]
       domain = options[:domain]
       domain = :local unless options[:suggest_alternate]
-      supress_suggestions = (domain == :local)
+      suppress_suggestions = (domain == :local)
 
       begin
         install_gem gem_name, gem_version
@@ -226,11 +225,11 @@ You can use `i` command instead of `install`.
         alert_error "Error installing #{gem_name}:\n\t#{e.message}"
         exit_code |= 1
       rescue Gem::GemNotFoundException => e
-        show_lookup_failure e.name, e.version, e.errors, supress_suggestions
+        show_lookup_failure e.name, e.version, e.errors, suppress_suggestions
 
         exit_code |= 2
       rescue Gem::UnsatisfiableDependencyError => e
-        show_lookup_failure e.name, e.version, e.errors, supress_suggestions,
+        show_lookup_failure e.name, e.version, e.errors, suppress_suggestions,
                             "'#{gem_name}' (#{gem_version})"
 
         exit_code |= 2
@@ -270,5 +269,4 @@ You can use `i` command instead of `install`.
     gems = @installed_specs.length == 1 ? 'gem' : 'gems'
     say "#{@installed_specs.length} #{gems} installed"
   end
-
 end
